@@ -4,7 +4,7 @@ use strict;
 use warnings;
 
 use vars qw($VERSION);
-$VERSION = '0.16';
+$VERSION = '0.25';
 
 #--------------------------------------------------------------------------
 
@@ -76,7 +76,7 @@ use HTML::TokeParser;
 #Constants                                                                #
 ###########################################################################
 
-use constant    UCC => 'http://www.xe.com/ucc';
+use constant    UCC => 'http://www.xe.com/currencyconverter';
 
 ###########################################################################
 #Variables                                                                #
@@ -86,7 +86,7 @@ my %currencies; # only need to load once!
 my @defaults = ('source', 'target', 'format');
 
 my $web = WWW::Mechanize->new();
-$web->agent_alias( 'Windows Mozilla' );
+$web->agent_alias( 'Mac Safari' );
 
 #--------------------------------------------------------------------------
 
@@ -132,8 +132,9 @@ Allows the user to add currencies to the internal hash. Currencies can be added
 as per the code below:
 
     $self->add_currencies(
-                    ZZZ => {text => 'An Example', symbol => '$'},
-                    ZZY => {text => 'Testing'} );
+                ZZZ => {text => 'An Example', symbol => '$'},
+                ZZY => {text => 'Testing'} 
+    );
 
 Note that unless otherwise stated, the symbol will be set to '&#164;'. The code
 used must be 3 characters in length, and a text part must be included.
@@ -237,18 +238,37 @@ sub convert {
 
     # get the base site
     $web->get( UCC );
+
     unless($web->success()) {
         $self->{error} = 'Unable to retrieve webpage';
         return;
     }
 
+	my @forms = $web->forms();
+	my $form_number = 1;
+	my $found = 0;
+
+	foreach my $form (@forms) {
+		if ($form->action eq 'http://www.xe.com/currencyconverter/convert/') {
+			$found = 1;
+			last;
+		}
+
+		$form_number++;
+	}
+
+	if ($found) {
     # complete and submit the form
     $web->submit_form(
-            form_name => 'ucc',
+            form_number => $form_number,
             fields    => { 'From'   => $params{source},
                            'To'     => $params{target},
-                           'Amount' => $params{value} } );
-    unless($web->success()) {
+                           'Amount' => $params{value}
+            }
+		);
+	}
+
+    unless($found && $web->success()) {
         $self->{error} = 'Unable to retrieve webform';
         return;
     }
@@ -283,7 +303,7 @@ sub _initialize {
     # Extract the mapping of currencies and their atrributes
     while(<Finance::Currency::Convert::XE::DATA>){
         s/\s*$//;
-        my ($code,$text,$symbol) = split "";
+        my ($code,$text,$symbol) = split /\|/;
         $currencies{$code}->{name} = $text;
         $currencies{$code}->{symbol} = $symbol;
     }
@@ -314,22 +334,24 @@ sub _extract_text {
     my $tag;
     my $p = HTML::TokeParser->new(\$html);
 
-    # look for the faq link
-    while(1) {
-        return  unless($tag = $p->get_tag('a'));
-        last    if(defined $tag->[1]{href} && $tag->[1]{href} =~ /faq/);
-    }
+    # first look for the 'td' element
+    while (1) {
+        return unless ($tag = $p->get_tag('td'));
+        next unless (defined($tag->[1]{'align'}) && ($tag->[1]{'align'} eq 'left'));
+        # this will probably be the value
+        my $value = $p->get_trimmed_text;
 
-    # jump to the next table
-    $tag = $p->get_tag('table');
-
-    # from there look for the target value
-    while($p->get_token) {
-        my $text = $p->get_trimmed_text;
-
-        if(my ($value) = $text =~ /([\d\.\,]+) $self->{code}/) {
-            $value =~ s/,//g;
-            return sprintf $self->{string}, $value;
+        # then make sure this has the 'span' with the target
+        # currency code
+        my $tag2 = $p->get_tag('span');
+        my $cd = $p->get_trimmed_text;
+        if (defined($tag2) && defined($tag2->[1]{'class'} && $tag2->[1]{class} eq 'uccResCde'
+)) {
+            if ($cd eq $self->{code}) {
+                # found it, return
+                $value =~ s/,//g;
+                return sprintf $self->{string}, $value;
+            }
         }
     }
 
@@ -372,8 +394,8 @@ Unicode? Let me know if there are.
 
 =head1 SEE ALSO
 
-  WWW::Mechanize
-  HTML::TokeParser
+L<HTML::TokeParser>, 
+L<WWW::Mechanize>
 
 =head1 SUPPORT
 
@@ -396,189 +418,182 @@ RT: L<http://rt.cpan.org/Public/Dist/Display.html?Name=Finance-Currency-Convert-
 
 =head1 COPYRIGHT
 
-  Copyright  2002-2008 Barbie for Miss Barbell Productions.
+  Copyright  2002-2011 Barbie for Miss Barbell Productions.
 
-  This library is free software; you can redistribute it and/or modify it under
-  the same terms as Perl itself, using the Artistic License.
-
-The full text of the licenses can be found in the Artistic file included with
-this distribution, or in perlartistic file as part of Perl installation, in
-the 5.8.1 release or later.
+  This module is free software; you can redistribute it and/or
+  modify it under the Artistic Licence v2.
 
 =cut
 
 #--------------------------------------------------------------------------
 
 __DATA__
-AEDUnited Arab Emirates, Dirham&#164;
-AFNAfghanistan, Afghani&#164;
-ALLAlbania, Lek&#164;
-AMDArmenia, Dram&#164;
-ANGNetherlands Antilles Guilder&#164;
-AOAAngola, Kwanza&#164;
-ARSArgentina, Peso&#164;
-AUDAustralia, Dollar$
-AWGAruba, Guilder&#164;
-AZNAzerbaijan, New Manat&#164;
-BAMBosnia and Herzegovina, Convertible Marka&#164;
-BBDBarbados, Dollar&#164;
-BDTBangladesh, Taka&#164;
-BGNBulgaria, Lev&#164;
-BHDBahrain, Dinar&#164;
-BIFBurundi, Franc&#164;
-BMDBermuda, Dollar&#164;
-BNDBrunei, Dollar&#164;
-BOBBolivia, Boliviano&#164;
-BRLBrazil, Real&#164;
-BSDBahamas, Dollar&#164;
-BTNBhutan, Ngultrum&#164;
-BWPBotswana, Pula&#164;
-BYRBelarus, Ruble&#164;
-BZDBelize, Dollar&#164;
-CADCanada, Dollar$
-CDFCongo/Kinshasa, Franc&#164;
-CHFSwitzerland, Franc&#164;
-CLPChile, Peso&#164;
-CNYChina, Yuan Renminbi&#164;
-COPColombia, Peso&#164;
-CRCCosta Rica, Colon&#164;
-CUCCuba, Convertible Peso&#164;
-CUPCuba, Peso&#164;
-CVECape Verde, Escudo&#164;
-CZKCzech Republic, Koruna&#164;
-DJFDjibouti, Franc&#164;
-DKKDenmark, Krone&#164;
-DOPDominican Republic, Peso&#164;
-DZDAlgeria, Dinar&#164;
-EEKEstonia, Kroon&#164;
-EGPEgypt, Pound&#164;
-ERNEritrea, Nakfa&#164;
-ETBEthiopia, Birr&#164;
-EUREuro&#8364;
-FJDFiji, Dollar&#164;
-FKPFalkland Islands, Pound&#164;
-GBPGreat Britain, Pound&#163;
-GELGeorgia, Lari&#164;
-GGPGuernsey, Pound&#164;
-GHSGhana, Cedi&#164;
-GIPGibraltar, Pound&#164;
-GMDGambia, Dalasi&#164;
-GNFGuinea, Franc&#164;
-GTQGuatemala, Quetzal&#164;
-GYDGuyana, Dollar&#164;
-HKDHong Kong, Dollar&#164;
-HNLHonduras, Lempira&#164;
-HRKCroatia, Kuna&#164;
-HTGHaiti, Gourde&#164;
-HUFHungary, Forint&#164;
-IDRIndonesia, Rupiah&#164;
-ILSIsrael, New Shekel&#8362;
-IMPIsle of Man, Pound&#164;
-INRIndia, Rupee&#8360;
-IQDIraq, Dinar&#164;
-IRRIran, Rial&#164;
-ISKIceland, Krona&#164;
-JEPJersey, Pound&#164;
-JMDJamaica, Dollar&#164;
-JODJordan, Dinar&#164;
-JPYJapan, Yen&#165;
-KESKenya, Shilling&#164;
-KGSKyrgyzstan, Som&#164;
-KHRCambodia, Riel&#164;
-KMFComoros, Franc&#164;
-KPWNorth Korea, Won&#164;
-KRWSouth Korea, Won&#8361;
-KWDKuwait, Dinar&#164;
-KYDCayman Islands, Dollar&#164;
-KZTKazakhstan, Tenge&#164;
-LAKLaos, Kip&#164;
-LBPLebanon, Pound&#164;
-LKRSri Lanka, Rupee&#164;
-LRDLiberia, Dollar&#164;
-LSLLesotho, Loti&#164;
-LTLLithuania, Litas&#164;
-LVLLatvia, Lat&#164;
-LYDLibya, Dinar&#164;
-MADMorocco, Dirham&#164;
-MDLMoldova, Leu&#164;
-MGAMadagascar, Ariary&#164;
-MKDMacedonia, Denar&#164;
-MMKBurma, Kyat&#164;
-MMKMyanmar, Kyat&#164;
-MNTMongolia, Tughrik&#164;
-MOPMacau, Pataca&#164;
-MROMauritania, Ouguiya&#164;
-MURMauritius, Rupee&#164;
-MVRMaldives, Rufiyaa&#164;
-MWKMalawi, Kwacha&#164;
-MXNMexico, Peso&#164;
-MYRMalaysia, Ringgit&#164;
-MZNMozambique, Metical&#164;
-NADNamibia, Dollar&#164;
-NGNNigeria, Naira&#164;
-NIONicaragua, Cordoba&#164;
-NOKNorway, Krone&#164;
-NPRNepal, Rupee&#164;
-NZDNew Zealand, Dollar&#164;
-OMROman, Rial&#164;
-PABPanama, Balboa&#164;
-PENPeru, Nuevo Sol&#164;
-PGKPapua New Guinea, Kina&#164;
-PHPPhilippines, Peso&#164;
-PKRPakistan, Rupee&#8360;
-PLNPoland, Zloty&#164;
-PYGParaguay, Guarani&#164;
-QARQatar, Riyal&#164;
-RONRomania, New Leu&#164;
-RSDSerbia, Dinar&#164;
-RUBRussia, Ruble&#164;
-RWFRwanda, Franc&#164;
-SARSaudi Arabia, Riyal&#164;
-SBDSolomon Islands, Dollar&#164;
-SCRSeychelles, Rupee&#164;
-SDGSudan, Pound&#164;
-SEKSweden, Krona&#164;
-SGDSingapore, Dollar&#164;
-SHPSaint Helena, Pound&#164;
-SKKSlovakia, Koruna&#164;
-SLLSierra Leone, Leone&#164;
-SOSSomalia, Shilling&#164;
-SPLSeborga, Luigino&#164;
-SRDSuriname, Dollar&#164;
-STDSo Tome and Principe, Dobra&#164;
-SVCEl Salvador, Colon&#164;
-SYPSyria, Pound&#164;
-SZLSwaziland, Lilangeni&#164;
-THBThailand, Baht&#3647;
-TJSTajikistan, Somoni&#164;
-TMMTurkmenistan, Manat&#164;
-TNDTunisia, Dinar&#164;
-TOPTonga, Pa'anga&#164;
-TRYTurkey, New Lira&#164;
-TTDTrinidad and Tobago, Dollar&#164;
-TVDTuvalu, Dollar&#164;
-TWDTaiwan, New Dollar&#164;
-TZSTanzania, Shilling&#164;
-UAHUkraine, Hryvna&#164;
-UGXUganda, Shilling&#164;
-USDUnited States, Dollar$
-UYUUruguay, Peso&#164;
-UZSUzbekistan, Som&#164;
-VEFVenezuela, Bolivar Fuerte&#164;
-VNDVietnam, Dong&#164;
-VUVVanuatu, Vatu&#164;
-WSTSamoa, Tala&#164;
-WSTWestern Samoa, Tala&#164;
-XAFCFA Communaut Financire Africaine BEAC Franc&#164;
-XAGSilver Ounce&#164;
-XAUGold Ounce&#164;
-XCDEast Caribbean Dollar&#164;
-XDRInternational Monetary Fund Special Drawing Right&#164;
-XOFCFA Communaut Financire Africaine BCEAO Franc&#164;
-XPDPalladium Ounce&#164;
-XPFComptoirs Franais du Pacifique Franc&#164;
-XPTPlatinum Ounce&#164;
-YERYemen, Rial&#164;
-ZARSouth Africa, Rand&#164;
-ZMKZambia, Kwacha&#164;
-ZWDZimbabwe, Dollar&#164;
+AED|United Arab Emirates, Dirham|&#164;
+AFN|Afghan Afghani|&#164;
+ALL|Albanian Lek|&#164;
+AMD|Armenian Dram|&#164;
+ANG|Netherlands Antilles Guilder|&#164;
+AOA|Angolan Kwanza|&#164;
+ARS|Argentine Peso|&#164;
+AUD|Australian Dollar|$
+AWG|Aruban or Dutch Guilder|&#164;
+AZN|Azerbaijani New Manat|&#164;
+BAM|Bosnian Convertible Marka|&#164;
+BBD|Barbadian or Bajan Dollar|&#164;
+BDT|Bangladeshi Taka|&#164;
+BGN|Bulgarian Lev|&#164;
+BHD|Bahraini Dinar|&#164;
+BIF|Burundian Franc|&#164;
+BMD|Bermudian Dollar|&#164;
+BND|Bruneian Dollar|&#164;
+BOB|Bolivian Boliviano|&#164;
+BRL|Brazilian Real|&#164;
+BSD|Bahamian Dollar|&#164;
+BTN|Bhutanese Ngultrum|&#164;
+BWP|Botswana Pula|&#164;
+BYR|Belarusian Ruble|&#164;
+BZD|Belizean Dollar|&#164;
+CAD|Canadian Dollar|$
+CDF|Congolese Franc|&#164;
+CHF|Switzerland, Franc|&#164;
+CLP|Chilean Peso|&#164;
+CNY|Chinese Yuan Renminbi|&#164;
+COP|Colombian Peso|&#164;
+CRC|Costa Rican Colon|&#164;
+CUC|Cuban Convertible Peso|&#164;
+CUP|Cuban Peso|&#164;
+CVE|Cape Verdean Escudo|&#164;
+CZK|Czech Koruna|&#164;
+DJF|Djiboutian Franc|&#164;
+DKK|Danish Krone|&#164;
+DOP|Dominican Peso|&#164;
+DZD|Algerian Dinar|&#164;
+EEK|Estonia, Kroon|&#164;
+EGP|Egyptian Pound|&#164;
+ERN|Eritrean Nakfa|&#164;
+ETB|Ethiopian Birr|&#164;
+EUR|Euro|&#8364;
+FJD|Fijian Dollar|&#164;
+FKP|Falkland Island Pound|&#164;
+GBP|Great Britain, Pound|&#163;
+GEL|Georgian Lari|&#164;
+GGP|Guernsey Pound|&#164;
+GHS|Ghanaian Cedi|&#164;
+GIP|Gibraltar Pound|&#164;
+GMD|Gambian Dalasi|&#164;
+GNF|Guinean Franc|&#164;
+GTQ|Guatemalan Quetzal|&#164;
+GYD|Guyanese Dollar|&#164;
+HKD|Hong Kong Dollar|&#164;
+HNL|Honduran Lempira|&#164;
+HRK|Croatian Kuna|&#164;
+HTG|Haitian Gourde|&#164;
+HUF|Hungarian Forint|&#164;
+IDR|Indonesian Rupiah|&#164;
+ILS|Israeli Shekel|&#8362;
+IMP|Isle of Man Pound|&#164;
+INR|Indian Rupee|&#8360;
+IQD|Iraqi Dinar|&#164;
+IRR|Iranian Rial|&#164;
+ISK|Icelandic Krona|&#164;
+JEP|Jersey Pound|&#164;
+JMD|Jamaican Dollar|&#164;
+JOD|Jordanian Dinar|&#164;
+JPY|Japanese Yen|&#165;
+KES|Kenyan Shilling|&#164;
+KGS|Kyrgyzstan, Som|&#164;
+KHR|Cambodian Riel|&#164;
+KMF|Comoran Franc|&#164;
+KPW|North Korea, Won|&#164;
+KRW|South Korea, Won|&#8361;
+KWD|Kuwait, Dinar|&#164;
+KYD|Caymanian Dollar|&#164;
+KZT|Kazakhstani Tenge|&#164;
+LAK|Laos, Kip|&#164;
+LBP|Lebanon, Pound|&#164;
+LKR|Sri Lanka, Rupee|&#164;
+LRD|Liberia, Dollar|&#164;
+LSL|Lesotho, Loti|&#164;
+LTL|Lithuania, Litas|&#164;
+LVL|Latvia, Lat|&#164;
+LYD|Libya, Dinar|&#164;
+MAD|Morocco, Dirham|&#164;
+MDL|Moldova, Leu|&#164;
+MGA|Madagascar, Ariary|&#164;
+MKD|Macedonia, Denar|&#164;
+MNT|Mongolia, Tughrik|&#164;
+MOP|Macau, Pataca|&#164;
+MRO|Mauritania, Ouguiya|&#164;
+MUR|Mauritius, Rupee|&#164;
+MVR|Maldives, Rufiyaa|&#164;
+MWK|Malawi, Kwacha|&#164;
+MXN|Mexico, Peso|&#164;
+MYR|Malaysia, Ringgit|&#164;
+MZN|Mozambique, Metical|&#164;
+NAD|Namibia, Dollar|&#164;
+NGN|Nigeria, Naira|&#164;
+NIO|Nicaragua, Cordoba|&#164;
+NOK|Norway, Krone|&#164;
+NPR|Nepal, Rupee|&#164;
+NZD|New Zealand, Dollar|&#164;
+OMR|Oman, Rial|&#164;
+PAB|Panama, Balboa|&#164;
+PEN|Peru, Nuevo Sol|&#164;
+PGK|Papua New Guinea, Kina|&#164;
+PHP|Philippines, Peso|&#164;
+PKR|Pakistan, Rupee|&#8360;
+PLN|Poland, Zloty|&#164;
+PYG|Paraguay, Guarani|&#164;
+QAR|Qatar, Riyal|&#164;
+RON|Romania, New Leu|&#164;
+RSD|Serbia, Dinar|&#164;
+RUB|Russia, Ruble|&#164;
+RWF|Rwanda, Franc|&#164;
+SAR|Saudi Arabia, Riyal|&#164;
+SBD|Solomon Islands, Dollar|&#164;
+SCR|Seychelles, Rupee|&#164;
+SDG|Sudan, Pound|&#164;
+SEK|Sweden, Krona|&#164;
+SGD|Singapore, Dollar|&#164;
+SHP|Saint Helena, Pound|&#164;
+SKK|Slovakia, Koruna|&#164;
+SLL|Sierra Leone, Leone|&#164;
+SOS|Somalia, Shilling|&#164;
+SPL|Seborga, Luigino|&#164;
+SRD|Suriname, Dollar|&#164;
+STD|So Tome and Principe, Dobra|&#164;
+SVC|Salvadoran Colon|&#164;
+SYP|Syria, Pound|&#164;
+SZL|Swaziland, Lilangeni|&#164;
+THB|Thailand, Baht|&#3647;
+TJS|Tajikistan, Somoni|&#164;
+TMM|Turkmenistan, Manat|&#164;
+TND|Tunisia, Dinar|&#164;
+TOP|Tonga, Pa'anga|&#164;
+TRY|Turkey, New Lira|&#164;
+TTD|Trinidad and Tobago, Dollar|&#164;
+TVD|Tuvalu, Dollar|&#164;
+TWD|Taiwan, New Dollar|&#164;
+TZS|Tanzania, Shilling|&#164;
+UAH|Ukraine, Hryvna|&#164;
+UGX|Uganda, Shilling|&#164;
+USD|United States, Dollar|$
+UYU|Uruguay, Peso|&#164;
+UZS|Uzbekistan, Som|&#164;
+VEF|Venezuela, Bolivar Fuerte|&#164;
+VND|Vietnam, Dong|&#164;
+VUV|Vanuatu, Vatu|&#164;
+XAF|Central African CFA Franc BEAC|&#164;
+XAG|Silver Ounce|&#164;
+XAU|Gold Ounce|&#164;
+XBT|Bitcoin|&#164;
+XCD|East Caribbean Dollar|&#164;
+XDR|IMF Special Drawing Rights|&#164;
+XOF|CFA Franc|&#164;
+XPD|Palladium Ounce|&#164;
+XPF|CFP Franc|&#164;
+XPT|Platinum Ounce|&#164;
+YER|Yemen, Rial|&#164;
+ZAR|South Africa, Rand|&#164;
+ZMK|Zambia, Kwacha|&#164;
+ZWD|Zimbabwe, Dollar|&#164;
